"""Models for Autarco."""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import date, datetime
from typing import Any

from mashumaro import DataClassDictMixin, field_options
from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin
from mashumaro.types import SerializationStrategy


class DateStrategy(SerializationStrategy):
    """Date serialization strategy to handle the date format."""

    def serialize(self, value: date) -> str:
        """Serialize date to string."""
        return value.strftime("%Y-%m-%d")

    def deserialize(self, value: str) -> date:
        """Deserialize string to date."""
        return date.fromisoformat(value)


@dataclass
class PowerResponse(DataClassORJSONMixin):
    """Object representing an Power Response model from the API."""

    inverters: dict[str, Inverter]
    stats: Stats


@dataclass
class EnergyResponse(DataClassORJSONMixin):
    """Object representing an Energy Response model response from the API."""

    stats: Stats


@dataclass
class AccountResponse(DataClassORJSONMixin):
    """Object representing an Account Response model from the API."""

    sites: list[AccountSite] = field(metadata=field_options(alias="data"))


@dataclass
class AccountSite(DataClassORJSONMixin):
    """Object representing an Account Site model response from the API."""

    site_id: int
    public_key: str
    system_name: str = field(metadata=field_options(alias="name"))
    retailer: str = field(metadata=field_options(alias="name_retailer"))
    health: str


@dataclass
class Inverter(DataClassORJSONMixin):
    """Object representing an Inverter model response from the API."""

    serial_number: str = field(metadata=field_options(alias="sn"))
    out_ac_power: int
    out_ac_energy_total: int
    grid_turned_off: bool
    health: str


@dataclass
class Solar(DataClassDictMixin):
    """Object representing a Solar model response from the API."""

    power_production: int = field(metadata=field_options(alias="pv_now"))
    energy_production_today: int = field(metadata=field_options(alias="pv_today"))
    energy_production_month: int = field(metadata=field_options(alias="pv_month"))
    energy_production_total: int = field(metadata=field_options(alias="pv_to_date"))


@dataclass
class Battery(DataClassDictMixin):
    """Object representing a Battery model response from the API."""

    # Power - Flow
    flow_now: int = field(metadata=field_options(alias="battery_now"))
    net_charged_now: int = field(
        metadata=field_options(alias="battery_net_charged_now")
    )
    state_of_charge: int = field(metadata=field_options(alias="battery_soc"))

    # Energy - Discharged
    discharged_today: int = field(
        metadata=field_options(alias="battery_discharged_today")
    )
    discharged_month: int = field(
        metadata=field_options(alias="battery_discharged_month")
    )
    discharged_total: int = field(
        metadata=field_options(alias="battery_discharged_to_date")
    )

    # Energy - Charged
    charged_today: int = field(metadata=field_options(alias="battery_charged_today"))
    charged_month: int = field(metadata=field_options(alias="battery_charged_month"))
    charged_total: int = field(metadata=field_options(alias="battery_charged_to_date"))


@dataclass
class Graphs(DataClassORJSONMixin):
    """Object representing Graphs model from the API."""

    pv_power: dict[str, dict[datetime, int | None]] | None = None
    pv_energy: dict[str, dict[date, int | None]] | None = None


@dataclass
class Stats(DataClassORJSONMixin):
    """Object representing the Stats model from the API."""

    graphs: Graphs
    kpis: dict[str, Any]

    @property
    def generate_power_stats_inverter(self) -> dict[str, list[dict[str, Any]]] | None:
        """Generate power statistics by inverter."""
        if self.graphs.pv_power:
            power_stats_by_inverter = {}
            for inverter_id, power_data in self.graphs.pv_power.items():
                stats_list = [
                    {"timestamp": timestamp, "power": power}
                    for timestamp, power in power_data.items()
                ]
                power_stats_by_inverter[inverter_id] = stats_list
            return power_stats_by_inverter
        return None

    @property
    def generate_energy_stats_inverter(self) -> dict[str, list[dict[str, Any]]] | None:
        """Generate energy statistics by inverter."""
        if self.graphs.pv_energy:
            energy_stats_by_inverter = {}
            for inverter_id, energy_data in self.graphs.pv_energy.items():
                stats_list = [
                    {"timestamp": date, "energy": energy}
                    for date, energy in energy_data.items()
                ]
                energy_stats_by_inverter[inverter_id] = stats_list
            return energy_stats_by_inverter
        return None


@dataclass
class Site(DataClassORJSONMixin):
    """Object representing an Site model response from the API."""

    # pylint: disable-next=too-few-public-methods
    class Config(BaseConfig):
        """Mashumaro configuration."""

        serialization_strategy = {date: DateStrategy()}  # noqa: RUF012
        serialize_by_alias = True

    public_key: str
    name: str
    address: Address

    # CO2 compensations
    co2_today: int
    co2_month: int
    co2_total: int = field(metadata=field_options(alias="co2_to_date"))

    # Energy consumption
    consumption_today: int
    consumption_month: int
    consumption_total: int = field(metadata=field_options(alias="consumption_to_date"))

    has_consumption_meter: bool
    has_battery: bool
    timezone: str
    created_at: date | None = field(
        metadata=field_options(alias="dt_created"), default=None
    )


@dataclass
class Address(DataClassORJSONMixin):
    """Object representing an Address model response from the API."""

    street: str | None = field(
        metadata=field_options(alias="address_line_1"), default=None
    )
    zip_code: str | None = field(metadata=field_options(alias="postcode"), default=None)
    city: str | None = field(default=None)
    country: str | None = field(default=None)



"""Fixtures for the Autarco tests."""

from collections.abc import AsyncGenerator

import pytest
from aiohttp import ClientSession

from autarco import Autarco


@pytest.fixture(name="autarco_client")
async def client() -> AsyncGenerator[Autarco, None]:
    """Return a Autarco client."""
    async with (
        ClientSession() as session,
        Autarco(
            email="test@autarco.com", password="energy", session=session
        ) as autarco_client,
    ):
        yield autarco_client



"""Asynchronous Python client for Autarco API."""

from __future__ import annotations

import asyncio
import json
import socket
from dataclasses import dataclass
from importlib import metadata
from typing import Any, Self

from aiohttp import BasicAuth, ClientError, ClientResponseError, ClientSession
from aiohttp.hdrs import METH_GET
from yarl import URL

from .exceptions import (
    AutarcoAuthenticationError,
    AutarcoConnectionError,
    AutarcoError,
)
from .models import (
    AccountResponse,
    AccountSite,
    Battery,
    EnergyResponse,
    Inverter,
    PowerResponse,
    Site,
    Solar,
    Stats,
)

VERSION = metadata.version(__package__)


@dataclass
class Autarco:
    """Main class for handling connections to Autarco."""

    email: str
    password: str

    request_timeout: float = 15.0
    session: ClientSession | None = None

    _close_session: bool = False

    async def _request(
        self,
        uri: str,
        *,
        method: str = METH_GET,
        params: dict[str, Any] | None = None,
    ) -> str:
        """Handle a request to the Autarco API.

        A generic method for sending/handling HTTP requests done against
        the Autarco API.

        Args:
        ----
            uri: Request URI, without '/', for example, 'status'.
            method: HTTP method to use.
            params: Query parameters to send with the request.

        Returns:
        -------
            The response data from the Autarco API.

        Raises:
        ------
            AutarcoAuthenticationError: If the email or password is invalid.
            AutarcoConnectionError: An error occurred while communicating
                with the Autarco API.
            AutarcoConnectionTimeoutError: A timeout occurred while communicating
                with the Autarco API.
            AutarcoError: Received an unexpected response from the
                Autarco API.

        """
        url = URL.build(
            scheme="https",
            host="my.autarco.com",
            path="/api/site/",
        ).join(URL(uri))

        headers = {
            "Accept": "application/json",
            "User-Agent": f"PythonAutarco/{VERSION}",
        }

        if self.session is None:
            self.session = ClientSession()
            self._close_session = True

        # Set basic auth credentials.
        auth = BasicAuth(self.email, self.password)

        try:
            async with asyncio.timeout(self.request_timeout):
                response = await self.session.request(
                    method,
                    url,
                    auth=auth,
                    headers=headers,
                    params=params,
                    ssl=True,
                )
                response.raise_for_status()
        except TimeoutError as exception:
            msg = "Timeout occurred while connecting to Autarco API"
            raise AutarcoConnectionError(msg) from exception
        except ClientResponseError as exception:
            if exception.status == 401:
                msg = "Authentication to the Autarco API failed"
                raise AutarcoAuthenticationError(msg) from exception
            msg = "Error occurred while communicating with the Autarco API"
            raise AutarcoConnectionError(msg) from exception
        except (ClientError, socket.gaierror) as exception:
            msg = "Error occurred while communicating with the Autarco API"
            raise AutarcoConnectionError(msg) from exception

        content_type = response.headers.get("Content-Type", "")
        text = await response.text()
        if "application/json" not in content_type:
            msg = "Unexpected response from the Autarco API"
            raise AutarcoError(
                msg,
                {"Content-Type": content_type, "response": text},
            )

        return text

    async def _get_combined_data(self, public_key: str) -> dict[str, Any]:
        """Get a combined dictionary with power and energy data from a site.

        Args:
        ----
            public_key: The public key from your site.

        Returns:
        -------
            A dictionary with combined power and energy data.

        """
        power_response = await self._request(f"{public_key}/kpis/power")
        energy_response = await self._request(f"{public_key}/kpis/energy")
        return {**json.loads(power_response), **json.loads(energy_response)}

    async def get_account(self) -> list[AccountSite]:
        """Get account with list of sites.

        Returns
        -------
            A list of Site objects.

        """
        response = await self._request("")
        return AccountResponse.from_json(response).sites

    async def get_inverters(self, public_key: str) -> dict[str, Inverter]:
        """Get a list of all used inverters.

        Args:
        ----
            public_key: The public key from the specific site.

        Returns:
        -------
            A list of Inverter objects.

        """
        response = await self._request(f"{public_key}/power")
        return PowerResponse.from_json(response).inverters

    async def get_power_statistics(
        self, public_key: str, query_range: str = "day"
    ) -> Stats:
        """Get the statistics of the inverters.

        Args:
        ----
            public_key: The public key from the specific site.
            query_range: The range of time to get the statistics for.

        Returns:
        -------
            A list of Inverter objects.

        """
        response = await self._request(f"{public_key}/power", params={"r": query_range})
        return PowerResponse.from_json(response).stats

    async def get_energy_statistics(
        self, public_key: str, query_range: str = "month"
    ) -> Stats:
        """Get the statistics of the inverters.

        Args:
        ----
            public_key: The public key from the specific site.
            query_range: The range of time to get the statistics for.

        Returns:
        -------
            A list of Inverter objects.

        """
        response = await self._request(
            f"{public_key}/energy", params={"r": query_range}
        )
        return EnergyResponse.from_json(response).stats

    async def get_solar(self, public_key: str) -> Solar:
        """Get information about the solar production from a site.

        Args:
        ----
            public_key: The public key from your site.

        Returns:
        -------
            An Solar object.

        """
        combined_data = await self._get_combined_data(public_key)
        return Solar.from_dict(combined_data)

    async def get_site(self, public_key: str) -> Site:
        """Get information about your system site.

        Args:
        ----
            public_key: The public key from your site.

        Returns:
        -------
            An Site object.

        """
        # Combine base site data with KPI energy data to enrich CO2/consumption fields
        site_response = await self._request(f"{public_key}/")
        energy_response = await self._request(f"{public_key}/kpis/energy")
        combined: dict[str, Any] = {
            **json.loads(site_response),
            **json.loads(energy_response),
        }
        return Site.from_json(json.dumps(combined))

    async def get_battery(self, public_key: str) -> Battery:
        """Get information about the battery from a site.

        Args:
        ----
            public_key: The public key from your site.

        Returns:
        -------
            An Battery object.

        """
        combined_data = await self._get_combined_data(public_key)
        return Battery.from_dict(combined_data)

    async def close(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> Self:
        """Async enter.

        Returns
        -------
            The Autarco object.

        """
        return self

    async def __aexit__(self, *_exc_info: object) -> None:
        """Async exit.

        Args:
        ----
            _exc_info: Exec type.

        """
        await self.close()



"""Auth tests for Autarcp."""

# pylint: disable=protected-access
import pytest
from aresponses import ResponsesMockServer

from autarco import Autarco
from autarco.exceptions import AutarcoAuthenticationError


async def test_authentication_error(
    aresponses: ResponsesMockServer,
    autarco_client: Autarco,
) -> None:
    """Test authentication error is handled correctly."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/test",
        "GET",
        aresponses.Response(status=401),
    )
    with pytest.raises(AutarcoAuthenticationError):
        assert await autarco_client._request("test")



"""Asynchronous Python client for the Autarco API."""

import asyncio

from autarco import AccountSite, Autarco


async def main() -> None:
    """Show example on getting Autarco - inverters data."""
    async with Autarco(
        email="test@autarco.com",
        password="password",
    ) as client:
        account_sites: list[AccountSite] = await client.get_account()
        inverters = await client.get_inverters(account_sites[0].public_key)
        power = await client.get_power_statistics(
            account_sites[0].public_key,
            query_range="day",
        )
        energy = await client.get_energy_statistics(
            account_sites[0].public_key,
            query_range="month",
        )

        print("--- INVERTERS ---")
        print(inverters)
        print()

        print("--- POWER STATS ---")
        print(power.generate_power_stats_inverter)
        print()

        print("--- ENERGY STATS ---")
        print(energy.generate_energy_stats_inverter)


if __name__ == "__main__":
    asyncio.run(main())



"""Asynchronous Python client for the Autarco API."""

import asyncio

from autarco import AccountSite, Autarco, Battery, Inverter, Site, Solar


async def main() -> None:
    """Test."""
    async with Autarco(
        email="test@autarco.com",
        password="password",
    ) as autarco:
        account_sites: list[AccountSite] = await autarco.get_account()

        inverters: dict[str, Inverter] = await autarco.get_inverters(
            account_sites[0].public_key
        )
        solar: Solar = await autarco.get_solar(account_sites[0].public_key)
        site: Site = await autarco.get_site(account_sites[0].public_key)

        print("--- ACCOUNT ---")
        print(account_sites)
        print()
        for item in account_sites:
            print(f"Site ID: {item.site_id}")
            print(f"Public Key: {item.public_key}")
            print(f"Name: {item.system_name}")
            print(f"Retailer: {item.retailer}")
            print(f"Health: {item.health}")
        print()

        print("--- INVERTER(S) ---")
        print(inverters)
        print()
        for inverter in inverters.values():
            print(f"Serial Number: {inverter.serial_number}")
            print(f"Out AC Power: {inverter.out_ac_power}")
            print(f"Out AC Energy Total: {inverter.out_ac_energy_total}")
            print(f"Grid Turned Off: {inverter.grid_turned_off}")
            print(f"Health: {inverter.health}")
        print()

        print("--- SOLAR ---")
        print(solar)
        print()
        print(f"Power Production: {solar.power_production}")
        print(f"Energy Production - Today: {solar.energy_production_today}")
        print(f"Energy Production - Month: {solar.energy_production_month}")
        print(f"Energy Production - Total: {solar.energy_production_total}")
        print()

        print("--- SITE ---")
        print(site)
        print()
        print(f"Address: {site.address}")
        print(f"Timezone: {site.timezone}")
        print(f"Created At: {site.created_at}")
        print(f"Consumption Meter: {site.has_consumption_meter}")
        print(f"Has Battery: {site.has_battery}")

        if site.has_battery:
            battery: Battery = await autarco.get_battery(account_sites[0].public_key)
            print()
            print("--- BATTERY ---")
            print(battery)


if __name__ == "__main__":
    asyncio.run(main())



"""Basic tests for the Autarco API."""

# pylint: disable=protected-access
import asyncio
from unittest.mock import patch

import pytest
from aiohttp import ClientError, ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from autarco import Autarco
from autarco.exceptions import (
    AutarcoConnectionError,
    AutarcoError,
)


async def test_json_request(
    aresponses: ResponsesMockServer,
    autarco_client: Autarco,
) -> None:
    """Test JSON response is handled correctly."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
        ),
    )
    response = await autarco_client._request("test")
    assert response is not None
    await autarco_client.close()


async def test_internal_session(aresponses: ResponsesMockServer) -> None:
    """Test internal session is created and closed."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
        ),
    )
    async with Autarco(email="test@autarco.com", password="energy") as client:
        await client._request("test")


async def test_timeout(aresponses: ResponsesMockServer) -> None:
    """Test request timeout from Autarco API."""

    # Faking a timeout by sleeping
    async def response_handler(_: ClientResponse) -> Response:
        await asyncio.sleep(0.2)
        return aresponses.Response(
            body="Goodmorning!",
        )

    aresponses.add(
        "my.autarco.com",
        "/api/site/test",
        "GET",
        response_handler,
    )

    async with ClientSession() as session:
        client = Autarco(
            email="test@autarco.com",
            password="energy",
            session=session,
            request_timeout=0.1,
        )
        with pytest.raises(AutarcoConnectionError):
            assert await client._request("test")


async def test_content_type(
    aresponses: ResponsesMockServer,
    autarco_client: Autarco,
) -> None:
    """Test request content type error from Autarco API."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "blabla/blabla"},
        ),
    )
    with pytest.raises(AutarcoError):
        assert await autarco_client._request("test")


async def test_client_error() -> None:
    """Test request client error from Autarco API."""
    async with ClientSession() as session:
        client = Autarco(email="test@autarco.com", password="energy", session=session)
        with (
            patch.object(
                session,
                "request",
                side_effect=ClientError,
            ),
            pytest.raises(AutarcoConnectionError),
        ):
            assert await client._request("test")


async def test_response_status_404(
    aresponses: ResponsesMockServer,
    autarco_client: Autarco,
) -> None:
    """Test HTTP 404 response handling."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/test",
        "GET",
        aresponses.Response(text="GIVE ME SOLARPOWER!", status=404),
    )
    with pytest.raises(AutarcoConnectionError):
        assert await autarco_client._request("test")



"""Asynchronous Python client for the Autarco API."""

import asyncio

from autarco import AccountSite, Autarco, Battery


async def main() -> None:
    """Show example on getting Autarco - battery data."""
    async with Autarco(
        email="test@autarco.com",
        password="password",
    ) as client:
        account_sites: list[AccountSite] = await client.get_account()
        battery: Battery = await client.get_battery(account_sites[0].public_key)
        print(battery)


if __name__ == "__main__":
    asyncio.run(main())



"""Test the Autarco models."""

from datetime import date

from aresponses import ResponsesMockServer
from syrupy.assertion import SnapshotAssertion

from autarco import (
    AccountSite,
    Autarco,
    Battery,
    DateStrategy,
    Inverter,
    Site,
    Solar,
    Stats,
)

from . import load_fixtures


async def test_get_inverters(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Inverter object."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/power",
        "GET",
        aresponses.Response(
            text=load_fixtures("power.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    inverters: dict[str, Inverter] = await autarco_client.get_inverters(
        public_key="fake_key"
    )
    assert inverters == snapshot


async def test_get_solar(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Solar object."""
    # Energy response
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/kpis/energy",
        "GET",
        aresponses.Response(
            text=load_fixtures("kpis_energy.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    # Power response
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/kpis/power",
        "GET",
        aresponses.Response(
            text=load_fixtures("kpis_power.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    solar: Solar = await autarco_client.get_solar(public_key="fake_key")
    assert solar == snapshot


async def test_get_battery(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Battery object."""
    # Energy response
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/kpis/energy",
        "GET",
        aresponses.Response(
            text=load_fixtures("battery/kpis_energy.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    # Power response
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/kpis/power",
        "GET",
        aresponses.Response(
            text=load_fixtures("battery/kpis_power.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    battery: Battery = await autarco_client.get_battery(public_key="fake_key")
    assert battery == snapshot


async def test_get_site(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Site object."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/",
        "GET",
        aresponses.Response(
            text=load_fixtures("site.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    # Energy response used to enrich Site with CO2/consumption
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/kpis/energy",
        "GET",
        aresponses.Response(
            text=load_fixtures("kpis_energy.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    site: Site = await autarco_client.get_site(public_key="fake_key")
    assert site == snapshot


async def test_get_old_site(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - old Site object."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/",
        "GET",
        aresponses.Response(
            text=load_fixtures("old_site.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    # Energy response used to enrich Site with CO2/consumption
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/kpis/energy",
        "GET",
        aresponses.Response(
            text=load_fixtures("kpis_energy.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    site: Site = await autarco_client.get_site(public_key="fake_key")
    assert site == snapshot


async def test_get_account(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Account object."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/",
        "GET",
        aresponses.Response(
            text=load_fixtures("account.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    account_sites: list[AccountSite] = await autarco_client.get_account()
    assert account_sites == snapshot
    assert account_sites[0].public_key == "site_key_1"
    assert account_sites[1].public_key == "site_key_2"


async def test_power_statistics(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Power statistics data."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/power",
        "GET",
        aresponses.Response(
            text=load_fixtures("power.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    power_stats: Stats = await autarco_client.get_power_statistics(
        public_key="fake_key", query_range="day"
    )
    generator = power_stats.generate_power_stats_inverter
    assert power_stats == snapshot
    assert generator == snapshot
    assert power_stats.generate_energy_stats_inverter is None


async def test_energy_statistics(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    autarco_client: Autarco,
) -> None:
    """Test request from a Autarco API - Energy statistics data."""
    aresponses.add(
        "my.autarco.com",
        "/api/site/fake_key/energy",
        "GET",
        aresponses.Response(
            text=load_fixtures("energy.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    energy_stats: Stats = await autarco_client.get_energy_statistics(
        public_key="fake_key", query_range="month"
    )
    generator = energy_stats.generate_energy_stats_inverter
    assert energy_stats == snapshot
    assert generator == snapshot
    assert energy_stats.generate_power_stats_inverter is None


def test_serialize_date() -> None:
    """Test the serialization of a date object."""
    test_date = date(2021, 8, 1)
    assert DateStrategy().serialize(test_date) == "2021-08-01"
